package com.android.cideasybridge;

import android.hardware.usb.UsbDevice;
import android.hardware.usb.UsbDeviceConnection;
import android.hardware.usb.UsbEndpoint;
import android.hardware.usb.UsbInterface;
import android.hardware.usb.UsbManager;
import android.os.AsyncTask;
import android.util.Log;
import android.content.Context;

import android.widget.TextView;
import android.widget.Toast;

import java.lang.reflect.Array;
import java.util.Arrays;

/**
 * Cid Easy Bridge Sample 2.4 - Multiple Device Support
 * @author Kybele
 * Created : jan 10 2022
 * Last Update : Mar 15 2025
 */


public class UsbDataBinder {

	private static final boolean enableCheckDigitControl  = true;

	private Context ctx;

	private static final String ackAnswer = "ACK\r\n";
	private static final String dckAnswer = "DCK\r\n";
	private static final String nakAnswer = "NAK\r\n";

	private static boolean _forceclaim = true;
	private static int _sleept = 100;
	private static int _timeout = 500;

	private UsbDevice mDevice;
	private UsbManager mUsbManager;
	private  UsbDeviceConnection mConnection;
	private  UsbInterface mIntf;
	private  UsbEndpoint rEndpoint;
	private  UsbEndpoint wEndpoint;

	private Thread _readingThread = null;

	private  char sPort = 0;
	private  String sDateTime = "";
	private  String sCaller = "";
	private  String sCallee = "";
	private  String sOther = "";

	public interface binderListener {
		public void onBinderLineEvent(char thePort, String theEvent);
		public void onBinderCliReceived(char thePort, String theDateTime, String theCaller, String theCallee);
	}

	private  binderListener listener;

	public void setBinderListener(binderListener listener) {
		this.listener = listener;
	}

	public UsbDataBinder(UsbManager manager, UsbDevice device) {
		mUsbManager = manager;
		mDevice = device;
		mIntf = mDevice.getInterface(0);
		rEndpoint = mIntf.getEndpoint(0);
		wEndpoint = mIntf.getEndpoint(1);
		mConnection = mUsbManager.openDevice(mDevice);
	}

	public void onDestroy() {
		if (mConnection != null) {
			mConnection.releaseInterface(mIntf);
			mConnection.close();
			mConnection = null;
		}
	}

	public void StartCidEasyThread(Context _ctx) {
		ctx = _ctx;
		this.listener = null;

		if (_readingThread == null) {
			_readingThread = new Thread(readerReceiver);
			_readingThread.start();
		}
	}

	private Runnable readerReceiver = new Runnable() {
		byte[] rBytes = new byte[64];
		public void run() {
			while (true) {
				try {
					if (mConnection.claimInterface(mIntf, _forceclaim)) {
						if (mConnection.bulkTransfer(rEndpoint, rBytes, rBytes.length, _timeout) > 0)
							analyzePackage(rBytes, mDevice.getDeviceName());
					} else {
						Log.d("CidEasyBridge", "CAN'T CLAIM USB");
					}
				} catch (Exception e) {
					Log.d("CEBridge - readReceiver",Log.getStackTraceString(e));
				}
				Sleep(_sleept);
			}
		}
	};

	private  void analyzePackage(byte[] bytes, String devPath) {
		try {
			final String strPackage = composeString(bytes);
			//Log.d("CidEasyBridge","RECEIVE ("+devPath+") <-- "+strPackage);

			if (strPackage.contains("ENQ") ) sendData(ackAnswer, devPath);
			else if (strPackage.contains("ETB")) sendData(ackAnswer, devPath);
			else if (strPackage.contains("STA")) echoLineEvent(strPackage);
			else {
					sendData(dckAnswer, devPath);
					if (testCliPackage(bytes))
						cliReceived((char) sPort, sDateTime, sCaller, sCallee);
				}
		} catch (Exception e) {
			Log.d("CEBridge - analyzePackage",Log.getStackTraceString(e));
		}
	}

	private  void sendData(String sendString, String devPath) {
		try {
			//Log.d("CidEasyBridge", "ANSWER ("+devPath+") -->  "+sendString);

			byte[] bytes = sendString.getBytes();
			if (mConnection.claimInterface(mIntf, _forceclaim)) {
				mConnection.bulkTransfer(wEndpoint, bytes, bytes.length, _timeout);
			}

		} catch (Exception e) {
			Log.d("CEBridge - sendData",Log.getStackTraceString(e));
		}
	}

	private  void echoLineEvent(String strPackage) {
		try {
			char thePort = strPackage.substring(4,5).charAt(0);
			String theEvent = strPackage.substring(6,strPackage.length()-2);

			if (listener != null) listener.onBinderLineEvent(thePort, theEvent);
		} catch (Exception e) {
			Log.d("CEBridge - echoLineEvent",Log.getStackTraceString(e));
		}
	}

	private  boolean testCliPackage(byte[] Package) {
		boolean res = false;
		try {
			byte[] portNames = { 'A','B','C','D','S' };
			int[] pckTypes = { 0x04, 0x80 };

			byte pPort = Package[0];
			int pType = Math.abs((int) Package[1]);
			int pLen  = Math.abs((int) Package[2]);

			if ((pLen > 0) && (pLen < 65)  && Arrays.binarySearch(portNames, pPort) >= 0 && Arrays.binarySearch(pckTypes, pType) >= 0) {
				if (pType == 0x80) { res = parseMDMF(Package); }
				if (pType == 0x04) { res = parseSDMF(Package); }
			}
		} catch (Exception e) {
			Log.d("CEBridge - testCliPackage",Log.getStackTraceString(e));
		}
		return res;
	}

	private static boolean testCheckDigit(byte[] inputReport)
	{
		int pLen = 0;
		int cDigit = 123;
		int pDigit = 0;

		try {
			pLen = inputReport[2] + 3;
			cDigit = inputReport[pLen] & 255;
			for (int i=1;i<pLen;i++) pDigit = (pDigit + Math.abs((int) inputReport[i] & 255)) & 255;
			pDigit = Math.abs((int) (0x100 - pDigit)) & 255;
		} catch (Exception e) {
			Log.d("CEBridge - testCheckDigit",Log.getStackTraceString(e));
		}

		return pDigit == cDigit;
	}

	private  boolean parseMDMF(byte[] Package) {
		int packlength = 0;
		int datalength = 0;
		char theChar = 0;
		int i;

		try {
			packlength = Package[2] + 4;
			sPort = (char) Package[0];
			sCaller = "";
			sCallee = "";
			sDateTime = "";
			sOther = "";

			if (packlength > Package.length) { packlength = Package.length; }

			if (packlength > 0) {
				i = 3;
				datalength = 0;
				while (i < Package.length)	{
					if (Package[i] == 1) // Date Field
					{
						i++;
						if (i < packlength - 1) {
							datalength = Package[i];
							if (datalength != 8) datalength = 8;
							if (datalength == 8) {
								while ((datalength > 0) && (datalength < packlength)) {
									i++;
									theChar = (char) Package[i];
									sDateTime = sDateTime + theChar;
									datalength--;
								}
							}
						}
					}
					else if (Package[i] == 2)  // Number Field
					{
						i++;
						if (i < packlength - 1) {
							datalength = Package[i];
							if (datalength > (packlength - i - 1)) datalength = (packlength - i - 1);
							while ((datalength > 0) && (datalength < packlength)) {
								i++;
								theChar = (char) Package[i];
								sCaller = sCaller + theChar;
								datalength--;
							}
						}
					}
					else if (Package[i] == 34)  //Callee field
					{
						// Other Fields
						i++;
						if (i < packlength - 1) {
							datalength = Package[i];
							while ((datalength > 0) && (datalength < packlength)) {
								i++;
								theChar = (char) Package[i];
								sCallee = sCallee + theChar;
								datalength--;
							}
						}
					}
					else
					{
						// Other Fields
						i++;
						if (i < packlength - 1) {
							datalength = Package[i];
							while ((datalength > 0) && (datalength < packlength)) {
								i++;
								theChar = (char) Package[i];
								sOther = sOther + theChar;
								datalength--;
							}
						}
					}
					i++;
				}
			}
		} catch (Exception e) {
			Log.d("CEBridge - parseMDMF",Log.getStackTraceString(e));
		}

		return (!enableCheckDigitControl || testCheckDigit(Package));
	}

	private  boolean parseSDMF(byte[] Package) {
		int packlength = 0;
		char theChar = 0;

		try {
			packlength = Package[2] + 4;
			sPort = (char)Package[0];
			sCaller = "";
			sCallee = "";
			sDateTime = "";
			sOther = "";

			for (int i = 3; i <= packlength - 2; i++) {
				if (i < Package.length) {
					theChar = (char) Package[i];
					if (i < 11) {
						sDateTime = sDateTime + (char) Package[i];
					} else {
						sCaller = sCaller + theChar;
					}
				}
			}
		} catch (Exception e) {
			Log.d("CEBridge - parsesDMF",Log.getStackTraceString(e));
		}

		return (!enableCheckDigitControl || testCheckDigit(Package));
	}

	private  void cliReceived(char Port, String DateTime, String Caller, String Callee) {
		try {
			if (listener != null)
				listener.onBinderCliReceived(Port, DateTime, Caller, Callee);
		} catch (Exception e) {
			Log.d("CEBridge - cliReceived",Log.getStackTraceString(e));
		}
	}

	private void Sleep(int milliseconds) {
		try {
			Thread.sleep(milliseconds);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}

	private  String composeString(byte[] bytes) {
		String strPackage = "";

		try {
			StringBuilder builder = new StringBuilder();
			for (byte b: bytes) {
				if (b > 0) {
					char c = (char) b;
					builder.append(c);
				}
			}
			strPackage = builder.toString();
		} catch (Exception e) {
			Log.d("CEBridge - composeString",Log.getStackTraceString(e));
		}

		return strPackage;
	}


}
